@@ -4,7 +4,7 @@ use warnings;
 
 package String::Tools;
 # ABSTRACT: Various tools for manipulating strings.
-our $VERSION = 'v0.18.270'; # VERSION
+our $VERSION = '0.18.277'; # VERSION
 
 
 use Exporter 'import';
@@ -16,7 +16,9 @@ our @EXPORT_OK = qw(
     shrink
     stitch
     stitcher
+    stringify
     subst
+    subst_vars
     trim
     trim_lines
 );
@@ -34,30 +36,31 @@ our $THREAD = ' ';
 
 ### Functions ###
 
+sub stringify(_);    # Forward declaration
+
 
 sub define(_) { return $_[0] // !!undef }
 
 
 sub is_blank(_) {
-    local $_ = shift;
-    return 1 unless defined() && length();
-    return /\A$BLANK+\z/;
+    local $_ = &stringify;
+    return not( length() && !/\A$BLANK+\z/ );
 }
 
 
 sub shrink(_) {
-    local $_ = trim(shift);
-    s/$BLANK+/$THREAD/g if defined;
+    local $_ = trim(&stringify);
+    s/$BLANK+/$THREAD/g;
     return $_;
 }
 
 
 sub stitch {
-    local $_;
-    my $str = '';
+    my $str       = '';
     my $was_blank = 1;
 
-    foreach my $s (map define, @_) {
+    local $_;
+    foreach my $s (map stringify, @_) {
         my $is_blank = is_blank($s);
         $str .= $THREAD unless ( $was_blank || $is_blank );
         $str .= $s;
@@ -74,16 +77,38 @@ sub stitcher {
 }
 
 
+sub stringify(_) {
+    local ($_) = @_;
+
+    if ( not defined() ) {
+        return define();
+    } elsif ( not( my $ref = ref() ) ) {
+        return $_;
+    } elsif ( $ref eq 'ARRAY' ) {
+        return "@$_";
+        #return join( $" // ' ', map stringify, @$_ );    # What about loops?
+    } elsif ( $ref eq 'HASH' ) {
+        return "@{[%$_]}";
+        #return join( $" // ' ', map stringify, %$_ );    # What about loops?
+    } elsif ( $ref eq 'REF' && ref($$_) ne 'REF' ) {
+        return stringify($$_);
+    } elsif ( $ref eq 'SCALAR' ) {
+        return stringify($$_);
+    }
+    return "$_";
+}
+
+
 sub subst {
-    my $str  = shift;
+    my $str  = stringify( shift );
     @_ = ( $_ ) if defined($_) && ! @_;
 
     my %subst;
     if ( 1 == @_ ) {
-        my $ref = ref $_[0];
-        if    ( $ref eq 'HASH' )  { %subst = %{ +shift }     }
-        elsif ( $ref eq 'ARRAY' ) { %subst = @{ +shift }     }
-        else                      { %subst = ( _ => +shift ) }
+        if ( not( my $ref = ref $_[0] ) ) { %subst = ( _ => +shift ) }
+        elsif ( $ref eq 'ARRAY' )         { %subst = @{ +shift }     }
+        elsif ( $ref eq 'HASH' )          { %subst = %{ +shift }     }
+        else                              { %subst = ( _ => +shift ) }
     } else { %subst = @_ }
 
     if (%subst) {
@@ -97,16 +122,25 @@ sub subst {
             )
             . ')\b';
         $str =~ s[\$(?:\{\s*($names)\s*\}|($names))]
-                 [$subst{ $1 // $2 }]g;
+                 [ stringify( $subst{ $1 // $2 } ) ]eg;
     }
 
     return $str;
 }
 
 
+sub subst_vars(_) {
+    local ($_) = &stringify;
+
+    my @vars = /\$(\{\s*$SUBST_VAR\s*\}|$SUBST_VAR\b)/g;
+    my %seen = ();
+    return grep { !$seen{$_}++ }
+        map { trim( $_, qr/\{\s*/, qr/\s*\}/ ) } @vars;
+}
+
+
 sub trim {
-    local $_ = @_ ? shift : $_;
-    return $_ unless defined;
+    local $_ = stringify( @_ ? shift : $_ );
 
     my ( $lead, $rear );
     my $count = scalar @_;
@@ -138,8 +172,7 @@ sub trim {
 
 
 sub trim_lines {
-    local $_ = @_ ? shift : $_;
-    return $_ unless defined;
+    local $_ = stringify( @_ ? shift : $_ );
 
     my ( $lead, $rear );
     my $count = scalar @_;
@@ -189,7 +222,9 @@ String::Tools - Various tools for manipulating strings.
     shrink
     stitch
     stitcher
+    stringify
     subst
+    subst_vars
     trim
     trim_lines
  );
@@ -334,6 +369,28 @@ L</$THREAD>.
  say stitcher( ' ' => $user, qw( home dir is /home/ ), '', $user );
  # "$user home dir is /home/$user"
 
+=head2 C<stringify( $scalar = $_ )>
+
+Return an intelligently stringified version of C<$scalar>.
+Attempts to avoid returning a string that has the reference name
+and a hexadecimal number:
+C<ARRAY(0xdeadbeef)>, C<My::Package=HASH(0xdeadbeef)>.
+
+If C<$scalar> is undefined,
+returns the result from L</define( $scalar = $_ )>.
+If C<$scalar> is not a reference,
+returns $scalar.
+If C<$scalar> is a reference to an C<ARRAY>,
+returns the stringification of that array (via C<"@$scalar">).
+If C<$scalar> is a reference to a C<HASH>,
+returns the stringification of that hash (via C<"@{[%$scalar]}">).
+If C<$scalar> is a reference to a C<SCALAR>,
+calls itself as C<stringify($$scalar)>.
+If C<$scalar> is a reference that is not one of the previously mentioned,
+returns the default stringification (via C<"$scalar">).
+
+Since v0.18.277
+
 =head2 C<< subst( $string ; %variables = ( _ => $_ ) ) >>
 
 Take in C<$string>, and do a search and replace of all the variables named in
@@ -357,6 +414,16 @@ The pattern is available in variable L</C<$SUBST_VAR>>.
 
 Returns the string with substitutions made.
 
+=head2 C<subst_vars( $string = $_ )>
+
+Search C<$string> for things that look like variables to be substituted.
+
+Returns the unique list of variable names found, without the leading
+C<$> or surrounding C<{}>.
+
+ my $string = 'Name is $name, age is $age, birthday is ${ birthday }';
+ my @vars = subst_vars($string);    # 'name', 'age', 'birtday'
+
 =head2 C<trim( $string = $_ ; $l = qr/$BLANK+/ ; $r = $l )>
 
 Trim C<string> of leading and trailing characters.
@@ -415,7 +482,7 @@ L<perlfunc/join>, Any templating system.
 
 =head1 VERSION
 
-version v0.18.270
+version 0.18.277
 
 =head1 AUTHOR
 
